// Copyright (c) 2024 Celestino Amoroso (celestino.amoroso@gmail.com).
// All rights reserved.

// operand-func.go
package expr

import (
	"errors"
	"fmt"
)

// -------- function call term
func newFuncCallTerm(tk *Token, args []*term) *term {
	return &term{
		tk:       *tk,
		class:    classVar,
		kind:     kindUnknown,
		parent:   nil,
		children: args,
		position: posLeaf,
		priority: priValue,
		evalFunc: evalFuncCall,
	}
}

// -------- eval func call
func evalFuncCall(ctx exprContext, self *term) (v any, err error) {
	name, _ := self.tk.Value.(string)
	params := make([]any, len(self.children))
	for i, tree := range self.children {
		var param any
		if param, err = tree.compute(ctx); err != nil {
			break
		}
		params[i] = param
	}
	if err == nil {
		v, err = ctx.Call(name, params)
	}
	return
}

// -------- function definition term
func newFuncDefTerm(tk *Token, args []*term) *term {
	return &term{
		tk:       *tk,
		class:    classVar,
		kind:     kindUnknown,
		parent:   nil,
		children: args, // arg[0]=formal-param-list, arg[1]=*ast
		position: posLeaf,
		priority: priValue,
		evalFunc: evalFuncDef,
	}
}

// -------- eval func def
// TODO
type funcDefFunctor struct {
	params []string
	expr   Expr
}

func (functor *funcDefFunctor) Invoke(parentCtx exprContext, name string, args []any) (result any, err error) {
	ctx := parentCtx.Clone()
	for i, p := range functor.params {
		if i < len(args) {
			ctx.SetValue(p, args[i])
		} else {
			ctx.SetValue(p, nil)
		}
	}
	result, err = functor.expr.Eval(ctx, false)
	return
}

func evalFuncDef(ctx exprContext, self *term) (v any, err error) {
	bodySpec := self.value()
	if expr, ok := bodySpec.(*ast); ok {
		paramList := make([]string, 0, len(self.children))
		for i, param := range self.children {
			if paramName, ok := param.value().(string); ok {
				paramList = append(paramList, paramName)
			} else {
				err = fmt.Errorf("invalid function definition: formal param nr %d must be an identifiers", i+1)
				break
			}
		}
		v = &funcDefFunctor{
			params: paramList,
			expr:   expr,
		}
	} else {
		err = errors.New("invalid function definition: the body specification must be an expression")
	}
	return
}
