// Copyright (c) 2024 Celestino Amoroso (celestino.amoroso@gmail.com).
// All rights reserved.

// operator-assign.go
package expr

//-------- assign term

func newAssignTerm(tk *Token) (inst *term) {
	return &term{
		tk:       *tk,
		class:    classOperator,
		kind:     kindUnknown,
		children: make([]*term, 0, 2),
		position: posInfix,
		priority: priAssign,
		evalFunc: evalAssign,
	}
}

func evalAssign(ctx exprContext, self *term) (v any, err error) {
	if err = self.checkOperands(); err != nil {
		return
	}

	leftTerm := self.children[0]
	if leftTerm.tk.Sym != SymIdentifier {
		err = leftTerm.tk.Errorf("left operand of %q must be a variable", self.tk.source)
		return
	}

	if v, err = self.children[1].compute(ctx); err == nil {
		if functor, ok := v.(Functor); ok {
			ctx.RegisterFunc(leftTerm.source(), functor, 0, -1)
		} else {
			ctx.SetValue(leftTerm.tk.source, v)
		}
	}
	return
}

// init
func init() {
	registerTermConstructor(SymEqual, newAssignTerm)
}
